// C++ Vererbung: Geom. Objekte
// Klasse Comp (zusammengesetzte Objekte), Implementierung
//
// Klaus Kusche, 2011

#include "comp.hxx"
#include "sdlinterf.h"

Comp::Comp(const Comp &orig) :
  BigObj(orig),  // Basisklassen-Konstruktor aufrufen
  mObjCnt(orig.mObjCnt), mMaxObjCnt(orig.mMaxObjCnt),
  mObj(new BigObj *[mMaxObjCnt]) {
    
  for (int i = 0; i < mObjCnt; ++i) {
    mObj[i] = orig.mObj[i]->clone();
  }
}

Comp::~Comp() {
  for (int i = 0; i < mObjCnt; ++i) {
    delete mObj[i];
  }
  delete[] mObj;
}

bool Comp::add(BigObj *obj) {
  if (mObjCnt >= mMaxObjCnt) return false;
  mObj[mObjCnt] = obj->clone();
  ++mObjCnt;
  return true;
}

void Comp::setPos(int x, int y) {
  move(x - mX, y - mY);
}

void Comp::setSize(int /*w*/, int /*h*/) {
  // ignorieren wir einfach, unsere BigObj haben keine Gre
}


void Comp::move(int x, int y) {
  mX += x;
  mY += y;
  for (int i = 0; i < mObjCnt; ++i) {
    mObj[i]->move(x, y);
  }
}

void Comp::scale(int percentX, int percentY) {
  for (int i = 0; i < mObjCnt; ++i) {
    mObj[i]->scale(percentX, percentX);
    int dX = ((mObj[i]->getX() - mX) * percentX) / 100;
    int dY = ((mObj[i]->getY() - mY) * percentY) / 100;
    mObj[i]->setPos(mX + dX, mY + dY);
  }
}


void Comp::rotate() {
  for (int i = 0; i < mObjCnt; ++i) {
    mObj[i]->rotate();
    int dX = mObj[i]->getX() - mX;
    int dY = mObj[i]->getY() - mY;
    mObj[i]->setPos(mX - dY, mY + dX);
  }
}

void Comp::draw() {
  for (int i = 0; i < mObjCnt; ++i) {
    mObj[i]->draw();
  }
}


void Comp::undraw() {
  for (int i = 0; i < mObjCnt; ++i) {
    mObj[i]->undraw();
  }
}
